package de.hendriklipka.pdfconv.gui;

import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.builder.PanelBuilder;

import javax.swing.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class ImageOptionsPage extends BaseWizardPage
{
    private JCheckBox _checkboxJPEG;
    private JTextField _textfieldQuality;
    private JCheckBox _checkboxBMP;
    private JCheckBox _checkboxBW;
    private JCheckBox _checkboxPICT;
    private JLabel _labelQuality;
    private JTextField _textfieldInternalScale;
    private JLabel _labelInternalScale;

    private boolean _allowed=false;
    private ActionListener _listener=new StateListener();
    private JTextField _textfieldWidth;
    private JTextField _textfieldHeigth;
    private JTextField _textfieldScale;

    public ImageOptionsPage(LogicHandler logicHandler)
    {
        _logicHandler=logicHandler;
    }

    protected void buildPanel()
    {
        FormLayout layout=new FormLayout("fill:100dlu,10dlu,right:max(40dlu;pref),3dlu,d,3dlu,fill:default:grow",
            "d,5,d,3,d,3,d,3,d,3,d,5,d,3,d,3,d,3,d,3,d,3,d,fill:0dlu:grow");

        PanelBuilder builder=new PanelBuilder(layout);
        builder.setDefaultDialogBorder();

        _panel=builder.getPanel();
        _panel.putClientProperty("jgoodies.noContentBorder", Boolean.TRUE);
        _panel.setBorder(BorderFactory.createEmptyBorder());

        CellConstraints cc=new CellConstraints();

        JTextArea jt=new JTextArea("Specify how images should be converted.\n\nIf required, specify the target formats. "+
            "If you want, you can specify how the images should be resized.");
        jt.setEditable(false);
        jt.setEnabled(true);
        jt.setWrapStyleWord(true);
        jt.setLineWrap(true);
        builder.add(jt, cc.xywh(1, 1, 1, 24));

        builder.addTitle("How should images be handled?", cc.xywh(3, 1, 5, 1));

        builder.addSeparator("Image types", cc.xywh(3, 3, 5, 1));
        _checkboxJPEG=new JCheckBox("JPEG");
        _checkboxJPEG.setFocusPainted(false);
        _checkboxPICT=new JCheckBox("PICT");
        _checkboxPICT.setFocusPainted(false);
        _checkboxBMP=new JCheckBox("BMP");
        _checkboxBMP.setFocusPainted(false);
        _checkboxBW = new JCheckBox("(Black&White)");
        _checkboxBW.setFocusPainted(false);
        _textfieldQuality=new JTextField();
        _labelQuality=new JLabel("Quality");

        _checkboxJPEG.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                if (_checkboxJPEG.isSelected())
                {
                    _textfieldQuality.setEnabled(true);
                    _labelQuality.setEnabled(true);
                }
                else
                {
                    _textfieldQuality.setEnabled(false);
                    _labelQuality.setEnabled(false);
                }
            }
        });
        _checkboxBMP.addActionListener(_listener);
        _checkboxJPEG.addActionListener(_listener);
        _checkboxPICT.addActionListener(_listener);

        _checkboxBMP.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                _checkboxBW.setEnabled(_checkboxBMP.isSelected());
            }
        });
        _checkboxBW.setEnabled(false);

        builder.add(_checkboxJPEG, cc.xy(3, 5, "left,center"));
        builder.add(_labelQuality, cc.xy(5, 5));
        builder.add(_textfieldQuality, cc.xy(7, 5));

        _textfieldQuality.setEnabled(false);
        _labelQuality.setEnabled(false);

        builder.add(_checkboxBMP, cc.xy(3, 7, "left,center"));
        builder.add(_checkboxBW, cc.xy(7, 7, "left,center"));
        builder.add(_checkboxPICT, cc.xy(3, 11, "left,center"));

        builder.addSeparator("Scaling", cc.xywh(3, 13, 5, 1));

        builder.addLabel("Scale", cc.xy(3, 15));
        _textfieldScale = new JTextField();
        builder.add(_textfieldScale, cc.xywh(5, 15, 3, 1));

        builder.addLabel("Max Width", cc.xy(3, 17));
        _textfieldWidth = new JTextField();
        builder.add(_textfieldWidth, cc.xywh(5, 17, 3, 1));

        builder.addLabel("Max Heigth", cc.xy(3, 19));
        _textfieldHeigth = new JTextField();
        builder.add(_textfieldHeigth, cc.xywh(5, 19, 3, 1));

        _labelInternalScale=new JLabel("Internal Scale");
        _textfieldInternalScale = new JTextField();
        builder.add(_labelInternalScale, cc.xy(3, 23));
        builder.add(_textfieldInternalScale, cc.xywh(5, 23, 3, 1));

    }

    public boolean allowFinish()
    {
        return _allowed;
    }

    public void leavePage()
    {
        _logicHandler.setConfigurationData(PageConstants.IMG_TYPE_BMP,
                                           _checkboxBMP.isSelected() ? (_checkboxBW.isSelected()?"false":"true") : "false");
        _logicHandler.setConfigurationData(PageConstants.IMG_TYPE_BMP2,
                                           _checkboxBMP.isSelected() ? (_checkboxBW.isSelected() ? "true" : "false") : "false");
        _logicHandler.setConfigurationData(PageConstants.IMG_TYPE_JPEG, _checkboxJPEG.isSelected() ? "true" : "false");
        _logicHandler.setConfigurationData(PageConstants.IMG_TYPE_PICT, _checkboxPICT.isSelected() ? "true" : "false");

        _logicHandler.setConfigurationData(PageConstants.IMG_QUALITY, _checkboxJPEG.isSelected()? _textfieldQuality.getText():"");
        _logicHandler.setConfigurationData(PageConstants.IMG_SCALE_FACTOR, _textfieldScale.getText());
        _logicHandler.setConfigurationData(PageConstants.IMG_SCALE_MAX_HEIGTH, _textfieldHeigth.getText());
        _logicHandler.setConfigurationData(PageConstants.IMG_SCALE_MAX_WIDTH, _textfieldWidth.getText());
        _logicHandler.setConfigurationData(PageConstants.IMG_SCALE_INTERNAL, _labelInternalScale.isVisible()?_textfieldInternalScale.getText():"");
    }

    public void setDesiredImageType(String type)
    {
        if (type.equalsIgnoreCase(PageConstants.IMG_TYPE_BMP))
        {
            _textfieldQuality.setVisible(false);
            _labelQuality.setVisible(false);
            _checkboxJPEG.setVisible(false);
            _checkboxJPEG.setSelected(false);
            _checkboxBMP.setVisible(true);
            _checkboxBMP.setEnabled(false);
            _checkboxBMP.setSelected(true);
            _checkboxBW.setVisible(true);
            _checkboxBW.setEnabled(true);
            _checkboxPICT.setVisible(false);
            _checkboxPICT.setSelected(false);
        }
        else if (type.equalsIgnoreCase(PageConstants.IMG_TYPE_JPEG))
        {
            _textfieldQuality.setVisible(true);
            _labelQuality.setVisible(true);
            _textfieldQuality.setEnabled(true);
            _labelQuality.setEnabled(true);
            _checkboxJPEG.setVisible(true);
            _checkboxJPEG.setEnabled(false);
            _checkboxBMP.setVisible(false);
            _checkboxPICT.setVisible(false);
            _checkboxBMP.setSelected(false);
            _checkboxJPEG.setSelected(true);
            _checkboxPICT.setSelected(false);
            _checkboxBW.setVisible(false);
            _checkboxBW.setEnabled(false);
        }
        else if (type.equalsIgnoreCase(PageConstants.IMG_TYPE_PICT))
        {
            _textfieldQuality.setVisible(false);
            _labelQuality.setVisible(false);
            _checkboxJPEG.setVisible(false);
            _checkboxBMP.setVisible(false);
            _checkboxPICT.setVisible(true);
            _checkboxPICT.setEnabled(false);
            _checkboxBMP.setSelected(false);
            _checkboxJPEG.setSelected(false);
            _checkboxPICT.setSelected(true);
            _checkboxBW.setVisible(false);
            _checkboxBW.setEnabled(false);
        }
        else
        {
            _textfieldQuality.setVisible(true);
            _labelQuality.setVisible(true);
            _checkboxJPEG.setVisible(true);
            _checkboxBMP.setVisible(true);
            _checkboxPICT.setVisible(true);
            _checkboxJPEG.setEnabled(true);
            _checkboxBMP.setEnabled(true);
            _checkboxPICT.setEnabled(true);
            _checkboxBW.setVisible(true);
            _checkboxBW.setEnabled(_checkboxBMP.isSelected());
            if (_checkboxJPEG.isSelected())
            {
                _textfieldQuality.setEnabled(true);
                _labelQuality.setEnabled(true);
            }
            else
            {
                _textfieldQuality.setEnabled(false);
                _labelQuality.setEnabled(false);
            }
        }
    }

    public void setAdvancedOptions(boolean advanced)
    {
        if (advanced)
        {
            _labelInternalScale.setVisible(true);
            _textfieldInternalScale.setVisible(true);
        }
        else
        {
            _labelInternalScale.setVisible(false);
            _textfieldInternalScale.setVisible(false);
        }
    }

    class StateListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            if (_checkboxBMP.isSelected())
                _allowed=true;
            else if (_checkboxBW.isSelected())
                _allowed = true;
            else if (_checkboxJPEG.isSelected())
                _allowed=true;
            else if (_checkboxPICT.isSelected())
                _allowed=true;
            else
                _allowed=false;
            _logicHandler.changeFinishState();
        }
    }
}
