package de.hendriklipka.pdfconv.gui;

import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.builder.PanelBuilder;

import javax.swing.*;
import javax.swing.event.CaretListener;
import javax.swing.event.CaretEvent;
import javax.swing.filechooser.FileFilter;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

public class SelectFilesPage extends BaseWizardPage
{
    JCheckBox _cbMultiple=null;
    JList _fileList=null;
    JButton _browseFile=null;
    JButton _removeFile=null;
    JLabel _labelFiles=null;
    JTextField _fileName=null;
    DefaultListModel _listData=null;
    private boolean _allowed=false;

    public SelectFilesPage(LogicHandler logicHandler)
    {
        _logicHandler=logicHandler;
        _enabled=true;
    }

    protected void buildPanel()
    {
        FormLayout layout=new FormLayout("fill:100dlu,10dlu,right:max(40dlu;pref),3dlu,fill:default:grow,3dlu,pref",
            "p,3,p,3,p,3,p,3,fill:pref:grow,3,p");

        PanelBuilder builder=new PanelBuilder(layout);
        builder.setDefaultDialogBorder();

        _panel=builder.getPanel();
        int[][] rowGroups={{1,3}};
        layout.setRowGroups(rowGroups);
        _panel.putClientProperty("jgoodies.noContentBorder", Boolean.TRUE);
        _panel.setBorder(BorderFactory.createEmptyBorder());

        CellConstraints cc=new CellConstraints();

        JTextArea jt=new JTextArea("Select which file(s) you want to convert.\n\nIf you want to convert multiple files, specify them and then add them to the file list.");
        jt.setEditable(false);
        jt.setEnabled(true);
        jt.setWrapStyleWord(true);
        jt.setLineWrap(true);
        builder.add(jt, cc.xywh(1, 1, 1, 11));

        builder.addTitle("Which files do you want to convert?", cc.xywh(3, 1, 5, 1));

        _fileName=new JTextField();
        builder.add(_fileName, cc.xy(5, 3));
        builder.addLabel("File name", cc.xy(3, 3));
        _fileName.setScrollOffset(3);
        _fileName.setAutoscrolls(true);
        _fileName.addCaretListener(new CaretListener()
        {
            public void caretUpdate(CaretEvent e)
            {
                setStates();
            }
        });

        _browseFile=new JButton("Browse...");
        builder.add(_browseFile, cc.xy(7, 3));

        _cbMultiple=new JCheckBox("Convert multiple files");
        _cbMultiple.setFocusPainted(false);
        builder.add(_cbMultiple, cc.xy(5, 5));

        _listData=new DefaultListModel();
        _fileList=new JList(_listData);
        builder.add(new JScrollPane(_fileList), cc.xywh(5, 7, 3, 3));
        _labelFiles=new JLabel("Files");
        builder.add(_labelFiles, cc.xy(3, 7));

        _removeFile=new JButton("Remove file");
        
        builder.add(_removeFile,cc.xy(7,11));

        _browseFile.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                browseForFile();
            }
        });
        
        _removeFile.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                removeFileFromList();
            }
        });
        _cbMultiple.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                if (_cbMultiple.isSelected())
                {
                    _fileList.setEnabled(true);
                    _labelFiles.setEnabled(true);
                    _browseFile.setText("Add...");
                    if (0==_listData.size())
                    {
                        addFileToList();
                    }
                    _fileName.setText("");
                    setStates();
                }
                else
                {
                    _fileList.setEnabled(false);
                    _removeFile.setEnabled(false);
                    _labelFiles.setEnabled(false);
                    _browseFile.setText("Browse...");
                    if (1==_listData.size())
                    {
                        _fileName.setText((String)_listData.elementAt(0));
                    }
                }
                setStates();
            }
        });
        _fileList.setEnabled(false);
        _removeFile.setEnabled(false);
        _labelFiles.setEnabled(false);
    }

    private void browseForFile()
    {
        String currentFileName=_fileName.getText();
        JFileChooser chooser=new JFileChooser(currentFileName);    
        chooser.setDialogTitle("Select PDF source file");
        chooser.setMultiSelectionEnabled(false);
        chooser.setFileFilter(new FileFilter()
        {
            public boolean accept(File f)
            {
                if (f.getName().toLowerCase().endsWith(".pdf"))
                    return true;
                if (f.isDirectory())
                    return true;
                return false;
            }

            public String getDescription()
            {
                return "PDF file";
            }
        });
        if (0==currentFileName.length())
        {
            File f=new File (".");
            chooser.setCurrentDirectory(f);
        }
        currentFileName="";
        int retVal=chooser.showOpenDialog(this._panel);
        try
        {
            if (retVal==JFileChooser.APPROVE_OPTION)
            {
                currentFileName=chooser.getSelectedFile().getCanonicalPath();
            }
        }
        catch (IOException e)
        {
        }
        _fileName.setText(currentFileName);
        if (_cbMultiple.isSelected())
        {
            addFileToList();
        }
    }

    private void addFileToList()
    {
        String currentFileName=_fileName.getText();
        if (null!=currentFileName&&0!=currentFileName.length())
        {
            _listData.addElement(currentFileName);
            setStates();
        }
        _fileName.setText("");
    }

    private void removeFileFromList()
    {
        int[] indices=_fileList.getSelectedIndices();
        for (int i=indices.length-1; i>=0; i--)
        {
            int index=indices[i];
            _listData.removeElementAt(index);
        }
        setStates();
    }

    public void leavePage()
    {
    }

    public boolean allowFinish()
    {
        return _allowed;
    }
    
    private void setStates()
    {
        if (_cbMultiple.isSelected())
        {
            if (0!=_listData.size())
            {
                _allowed=true;
                _removeFile.setEnabled(false);
            }
            else
            {
                _allowed=false;
            }
        }
        else
        {
            if (0!=_fileName.getText().length())
            {
                _allowed=true;
            }
            else
            {
                _allowed=false;
            }
            
        }
        _logicHandler.changeFinishState();
    }

    public Enumeration getFiles()
    {
        if (_cbMultiple.isSelected()&&0!=_listData.size())
            return _listData.elements();
        Vector v=new Vector();
        v.addElement(_fileName.getText());
        return v.elements();
    }
}
