package de.hendriklipka.pdfconv.gui;

import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.ButtonBarFactory;

import javax.swing.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.*;

public class WizardComponentImpl implements WizardComponent
{
    private JButton _backButton=new JButton("Back");
    private JButton _nextButton=new JButton("Next");
    private JButton _finishButton=new JButton("Finish");
    private JButton _cancelButton=new JButton("Cancel");

    private List _pages=new ArrayList();
    private int _currentPage=0;

    private JFrame _mainFrame=null;
    private LogicHandler _handler=null;

    public WizardComponentImpl()
    {
        _mainFrame=new JFrame();
        _mainFrame.setTitle("PDFConv");
        _mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    public void display()
    {
        JComponent panel=buildPanel();
        _mainFrame.getContentPane().add(panel);
        _mainFrame.pack();

        for (int i=1; i<_pages.size(); i++)
        {
            JPanel aPanel=((WizardPage)_pages.get(i)).getPanel();
            aPanel.setVisible(false);
            aPanel.setEnabled(false);
        }

        _backButton.setEnabled(false);

        _backButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                prevPage();
            }
        });

        _nextButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                nextPage();
            }
        });
        
        _cancelButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                _handler.cancelClicked();
            }
        });
        
        _finishButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                _handler.finishClicked();
            }
        });

        _handler.changeFinishState();
        _mainFrame.show();
    }

    private JComponent buildPanel()
    {
        FormLayout layout=new FormLayout("fill:default:grow", "fill:default:grow, 7dlu ,pref, 7dlu, default");
        PanelBuilder builder=new PanelBuilder(layout);
        builder.setDefaultDialogBorder();
        CellConstraints cc=new CellConstraints();

        for (Iterator iterator=_pages.iterator(); iterator.hasNext();)
        {
            WizardPage page=(WizardPage)iterator.next();
            JPanel panel=page.getPanel();
            builder.add(panel, cc.xy(1, 1, "fill,fill"));
        }

        builder.addSeparator("", cc.xywh(1, 3, 1, 1));

        JPanel buttonPanel=ButtonBarFactory.buildWizardBar(_backButton, _nextButton, _finishButton, _cancelButton);
        builder.add(buttonPanel, cc.xy(1, 5));

        return builder.getPanel();
    }

    public void addPage(WizardPage page)
    {
        _pages.add(page);
    }

    private void nextPage()
    {
        // already last page?
        if (!existLaterPages())
            return;
        WizardPage oldPage=(WizardPage)_pages.get(_currentPage);
        oldPage.leavePage();

        int i=_currentPage;

        while (++i<_pages.size())
        {
            WizardPage page=(WizardPage)_pages.get(i);
            if (page.isEnabled())
            {
                oldPage.getPanel().setVisible(false);
                oldPage.getPanel().setEnabled(false);
                _currentPage=i;
                WizardPage newPage=(WizardPage)_pages.get(_currentPage);
                newPage.getPanel().setVisible(true);
                newPage.getPanel().setEnabled(true);
                break;
            }
        }
        if (!existLaterPages())
        {
            _nextButton.setEnabled(false);
        }
        else
        {
            _nextButton.setEnabled(true);
        }
        _backButton.setEnabled(true);
        _handler.changeFinishState();
    }

    private boolean existLaterPages()
    {
        if (_currentPage==(_pages.size()-1))
            return false;

        for (int i=_currentPage+1; i<_pages.size(); i++)
        {
            WizardPage page=(WizardPage)_pages.get(i);
            if (page.isEnabled())
                return true;
        }
        return false;
    }

    private void prevPage()
    {
        // already last page?
        if (!existEarlierPages())
            return;
        WizardPage oldPage=(WizardPage)_pages.get(_currentPage);
        oldPage.leavePage();

        int i=_currentPage;

        while (--i>=0)
        {
            WizardPage page=(WizardPage)_pages.get(i);
            if (page.isEnabled())
            {
                oldPage.getPanel().setVisible(false);
                oldPage.getPanel().setEnabled(false);
                _currentPage=i;
                WizardPage newPage=(WizardPage)_pages.get(_currentPage);
                newPage.getPanel().setVisible(true);
                newPage.getPanel().setEnabled(true);
                break;
            }
        }
        if (!existEarlierPages())
        {
            _backButton.setEnabled(false);
        }
        else
        {
            _backButton.setEnabled(true);
        }
        _nextButton.setEnabled(true);
        _handler.changeFinishState();
    }

    private boolean existEarlierPages()
    {
        if (0==_currentPage)
            return false;

        int i=_currentPage-1;
        while (i>=0)
        {
            WizardPage page=(WizardPage)_pages.get(i);
            if (page.isEnabled())
                return true;
            i--;
        }
        return false;
    }

    public void rebuildLayout()
    {
        for (Iterator iterator=_pages.iterator(); iterator.hasNext();)
        {
            WizardPage page=(WizardPage)iterator.next();
            page.getPanel().setVisible(true);
        }
        Dimension size=_mainFrame.getSize();
        _mainFrame.setSize(size);
        _mainFrame.pack();
        _mainFrame.setSize(size);
        int i=0;
        for (Iterator iterator=_pages.iterator(); iterator.hasNext();)
        {
            WizardPage page=(WizardPage)iterator.next();
            if (i!=_currentPage)
                page.getPanel().setVisible(false);
            i++;
        }
    }

    public void changeFinishState(boolean allowed)
    {
        _finishButton.setEnabled(allowed);
    }

    public void setLogicHandler(LogicHandler handler)
    {
        _handler=handler;
    }

    public void gotoPage(WizardPage page)
    {
        WizardPage oldPage = (WizardPage) _pages.get(_currentPage);
        oldPage.leavePage();

        for (Iterator iterator=_pages.iterator(); iterator.hasNext();)
        {
            WizardPage wizardPage=(WizardPage)iterator.next();
            if (page.equals(wizardPage))
            {
                wizardPage.getPanel().setVisible(true);
                System.out.println("enabling page "+wizardPage);
            }
            else
            {
                wizardPage.getPanel().setVisible(false);
                System.out.println("disabling page "+wizardPage);
            }
        }
    }

    public void goIntoFinalState()
    {
        _backButton.setEnabled(false);
        _nextButton.setEnabled(false);
        _finishButton.setEnabled(false);
        _cancelButton.setEnabled(true);
        System.out.println("final state");
    }

    public void goIntoFinishedState()
    {
        _backButton.setEnabled(false);
        _nextButton.setEnabled(false);
        _finishButton.setEnabled(false);
        _cancelButton.setEnabled(true);
        _cancelButton.setText("Close");
        System.out.println("finished state");
    }
}
